using System;
using System.Drawing;
using IMusicCommon;
using IMelody;

namespace INoteLines { 
 public class NoteLines{

     public float Width;
     public float Height;
     public float offsX;
     public float offsY;

     public float MaxLineOffs = 5.5f;
     public float MinLineOffs = -7.0f;
     public float NoteRadiusPerc = 0.45f;

     public int TactNum = 4;

     //private int MaxMidiNote;
     //private int MinMidiNote;
     //public Brush brNoteBlack;
     public Brush brWhite;
     //public Pen pBlack;

     // Constructor
     public NoteLines(float tWidth, float tHeight) {
         Width = tWidth;
         Height = tHeight;

         //brNoteBlack = new SolidBrush(Color.Black);
         brWhite = new SolidBrush(Color.White);
         //pBlack = new Pen(Color.Black);
     }

     // 
     public void DrawToBMP(Graphics gBMP) {

         gBMP.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;

         //float ZeroLinePos = GetZeroLinePos();
         float LineStep = GetLineStep();

         gBMP.FillRectangle(brWhite, offsX, offsY, Width, Height);

         // Main Lines
         for (float LineNum = -2f; LineNum <= 2f; LineNum++)
         {
             float Ypos = GetLinePos(LineNum);
             gBMP.DrawLine(Pens.Black, offsX , Ypos, Width, Ypos);
         };

         

     } // end of 

     // Draw Note
     //public void DrawNote(Graphics e, int midinote, int timepos, float divider,  int direction = 0, int dot = 0 ) {
     public void DrawNote(Graphics e, Melody.MelodyNote note, Color notecolor, int t_xpos = 0)
     {

         int XPos = note.CaptureXpos;
         if (t_xpos != 0)
         {
             XPos = t_xpos;
         };

         if (note.MidiNote > 127) {
             DrawOtherItems(e, note, notecolor,XPos);
             return;
         };
         
         float NoteLine = MusicCommon.MidiNoteToLinePosition(note.MidiNote);
         float NotePosY = GetLinePos(NoteLine);
         float NoteRadius = GetLineStep() * NoteRadiusPerc;
         string NoteName = MusicCommon.MidiNoteToName(note.MidiNote);

         //brNoteBlack = new SolidBrush(Color.Black);

         Pen notePen = new Pen(notecolor);
         Brush noteBrush = new SolidBrush(notecolor);

         notePen.Width = NoteRadius * 0.3f;
         float LineLen = NoteRadius * 5;

         int timepos = XPos - (int)offsX;
         float divider = note.Divider;
         int direction = note.Direction;

         
         // print note name
         e.DrawString(NoteName, SystemFonts.MessageBoxFont, Brushes.Blue, offsX + timepos + 10, NotePosY);

         // Note Ellipse
         e.FillEllipse(noteBrush, offsX + timepos - NoteRadius, NotePosY - NoteRadius + (NoteRadius * 0.2f), NoteRadius * 2, NoteRadius * 2 - ((NoteRadius * 0.2f) * 2));
         if (divider == 1 || divider == 2) // for full or 1/2 note
         {
             e.FillEllipse(brWhite, offsX + timepos - (NoteRadius / 2), NotePosY - (NoteRadius / 2) + (NoteRadius * 0.15f), NoteRadius /* * 2*/ , NoteRadius - ((NoteRadius * 0.15f) * 2));
         };
         
         // Note Vertical line
         if (divider >= 2)
         {
             if(direction == 0)
             {// up line
                 e.DrawLine(notePen, offsX + timepos + NoteRadius - 1, NotePosY, offsX + timepos + NoteRadius - 1, NotePosY - LineLen);
             }
             else{// down line
                 e.DrawLine(notePen, offsX + timepos - NoteRadius + 1, NotePosY, offsX + timepos - NoteRadius + 1, NotePosY + LineLen);
             };
         };
         
         
         // 1/8 - 1/64 lines
         if (divider > 4) 
         {
             for (int i = 0; Math.Pow(2, i + 3)   <= divider; i ++) {
                 if (direction == 0)
                 {// up line
                     e.DrawLine(notePen, offsX + timepos + NoteRadius - 1, NotePosY - LineLen + (NoteRadius * i), offsX + timepos + NoteRadius - 1 + (NoteRadius * 1.5f), NotePosY - LineLen + (NoteRadius * 2f) + (NoteRadius * i));
                 }
                 else
                 {// down line
                     e.DrawLine(notePen, offsX + timepos - NoteRadius + 1, NotePosY + LineLen - (NoteRadius * i), offsX + timepos - NoteRadius - 1 - (NoteRadius * 1.5f), NotePosY + LineLen - (NoteRadius * 2f) - (NoteRadius * i));
                 };
                
             };
         };


         //   draw diez #

           // vertical
         if (MusicCommon.MidiNoteModifier(note.MidiNote) == "#")
         {
             float DiezCenterOffs = NoteRadius * 2;
             float DiezLinesOffs = NoteRadius / 3f;

             e.DrawLine(notePen, offsX + timepos - DiezCenterOffs + DiezLinesOffs, NotePosY - NoteRadius, offsX + timepos - DiezCenterOffs + DiezLinesOffs, NotePosY + NoteRadius);
             e.DrawLine(notePen, offsX + timepos - DiezCenterOffs - DiezLinesOffs, NotePosY - NoteRadius + DiezLinesOffs, offsX + timepos - DiezCenterOffs - DiezLinesOffs, NotePosY + NoteRadius + DiezLinesOffs);
             //horisontal
             e.DrawLine(notePen, offsX + timepos - DiezCenterOffs - DiezLinesOffs * 2, NotePosY - DiezLinesOffs / 2, offsX + timepos - DiezCenterOffs + DiezLinesOffs * 2, NotePosY - DiezLinesOffs * 2);
             e.DrawLine(notePen, offsX + timepos - DiezCenterOffs - DiezLinesOffs * 2, NotePosY - DiezLinesOffs / 2 + (DiezLinesOffs * 3), offsX + timepos - DiezCenterOffs + DiezLinesOffs * 2, NotePosY - DiezLinesOffs * 2 + (DiezLinesOffs * 3));
         };
         
         // dot
         if (note.Dot != 0)
         {
             e.FillEllipse(noteBrush, offsX + timepos + (NoteRadius * 1.5f), NotePosY + (NoteRadius / 2.5f), NoteRadius / 2, NoteRadius / 2);
         };


         // check for additional lines
        
         {
             for (float LineNum = 3f; LineNum <= NoteLine; LineNum++)
             {
                 float Ypos = GetLinePos(LineNum);
                 e.DrawLine(Pens.Black, offsX + (timepos - NoteRadius * 2), Ypos, offsX + (timepos + NoteRadius * 2), Ypos);
             };
         };
         if (NoteLine < 2)
         {
             for (float LineNum = -3f; LineNum >= NoteLine; LineNum--)
             {
                 float Ypos = GetLinePos(LineNum);
                 e.DrawLine(Pens.Black, offsX + (timepos - NoteRadius * 2), Ypos, offsX + (timepos + NoteRadius * 2), Ypos);
             };
         };
         // end of additional lines



     } // end of drawTo BMP

     public void DrawOtherItems(Graphics e, Melody.MelodyNote note, Color notecolor, int XPos){
         Pen nPen = new Pen(notecolor);

         switch (note.MidiNote){
             case 1000:   // tact line
                 e.DrawLine(nPen, XPos, GetLinePos(-2f), XPos, GetLinePos(2f));
                 break;
             case 1001:   // tact line
                 nPen.Width = 10;
                 nPen.Color = Color.FromArgb(128, 128, 255, 128);
                 e.DrawLine(nPen, XPos, GetLinePos(-6f), XPos, GetLinePos(7f));
                 break;
             case 1002:
                 nPen.Width = 10;
                 nPen.Color = Color.FromArgb(128, 128, 128, 255);
                 e.DrawLine(nPen, XPos, GetLinePos(-5f), XPos, GetLinePos(6f));
                 break;
                 
          
         };
     
     }

     //-------------------------------------------------------
     // common calculations
     public float GetZeroLinePos() {
         return this.GetLineStep() * Math.Abs(MinLineOffs);
     }

     public float GetLineStep() {
         int NumLines = (int)(Math.Abs(MaxLineOffs) + Math.Abs(MinLineOffs) + 1f);
         return Height / NumLines;
     }

     public float GetLinePos(float LineNum) { 
         float ZeroLinePos = GetZeroLinePos();
         float LineStep = GetLineStep();
         return  offsY + (ZeroLinePos + (LineNum * LineStep));
     }
     //-------------------------------------------------------
     // extenral interfaces
     public int GetMidiNoteAtCursor(int x, int y) {
         if (y < offsY || y > (offsY + Height)) { // above or below note grid
             return 0;
         };
         float Ypos = y - offsY;
         float ZeroLinePos = GetZeroLinePos();
         float NearestLine = ((float)Math.Round( ((Ypos - ZeroLinePos) * 2) / GetLineStep()))/2;
         
         if(NearestLine > MaxLineOffs || NearestLine < MinLineOffs){
             return 0;
         };

         return MusicCommon.LinePositionToMidiNote(NearestLine);
     }



     public float GetTimeOffsetAtCursor(int x, int y) {

         return (float)x;
         
     }

 }
}
